import os
import random
import string
import uuid
from typing import Callable

import pytest

from tests.integration_tests.configuration.client import create_client
from tests.integration_tests.configuration.loader import CONFIGURATION
from vantage_sdk.client import VantageClient


if CONFIGURATION["other"]["is_mock_api"]:
    from tests.integration_tests.configuration.mock_api import setup_mock

    setup_mock(CONFIGURATION["api"]["api_host"])


auth_method = CONFIGURATION["auth"]["auth_method"]
jwt_token = CONFIGURATION["auth"]["jwt_token"]

_client = create_client(
    auth_method=auth_method,
    jwt_token=jwt_token,
    configuration=CONFIGURATION,
)

_protected_collections = []

embedding_collection_id = CONFIGURATION["collection"].get(
    "embedding_search_test_collection_id"
)
semantic_colection_id = CONFIGURATION["collection"].get(
    "semantic_search_test_collection_id"
)
mlt_collection_id = CONFIGURATION["collection"].get(
    "more_like_this_collection"
)
if embedding_collection_id:
    _protected_collections.append(embedding_collection_id)
if semantic_colection_id:
    _protected_collections.append(semantic_colection_id)
if mlt_collection_id:
    _protected_collections.append(mlt_collection_id)


def skip_delete_external_api_key_test() -> bool:
    """
    Used to determine if TestApiKeys::test_delete_external_api_key test
    should run.
    """
    value = os.getenv("ENABLE_DELETE_KEY_TEST")

    return value is None or value.lower() not in ['true']


# Runs after all tests have finished
def pytest_sessionfinish(session, exitstatus):
    try:
        collections = _client.list_collections()
        for collection in collections:
            collection_id = collection.collection_id

            if collection_id in _protected_collections:
                continue

            _client.delete_collection(collection_id=collection_id)
    except Exception:
        # Do nothing
        pass

    if not CONFIGURATION["other"]["enable_external_api_tests"]:
        return

    try:
        keys = _client.get_external_keys()
        for key in keys:
            _client.delete_external_key(key.external_key_id)
    except Exception:
        # Do nothing
        pass


def random_string(length: int):
    letters = string.ascii_lowercase
    return ''.join(random.choice(letters) for i in range(length))


@pytest.fixture(scope="module")
def client() -> VantageClient:
    return _client


@pytest.fixture(scope="module")
def random_string_generator() -> Callable:
    return random_string


@pytest.fixture(scope="module")
def random_uuid() -> str:
    return str(uuid.uuid4())


def disable_external_api_keys_tests() -> bool:
    return not CONFIGURATION["other"]["is_mock_api"]



import pytest

from tests.integration_tests.configuration.loader import CONFIGURATION


@pytest.fixture(scope="module")
def api_params() -> dict:
    return CONFIGURATION["api"]


@pytest.fixture(scope="module")
def mock_api() -> dict:
    return CONFIGURATION["other"]["is_mock_api"]



#!/usr/bin/env python

import time
from pathlib import Path
from typing import Callable

import pytest

from tests.integration_tests.utilities import (
    create_temporary_collection,
    create_temporary_upe_collection,
)
from vantage_sdk.client import VantageClient
from vantage_sdk.core.exceptions import VantageFileUploadError
from vantage_sdk.core.http.exceptions import NotFoundException
from vantage_sdk.model.collection import (
    OpenAICollection,
    UserProvidedEmbeddingsCollection,
)


class TestCollections:
    """Integration tests for collection endpoints."""

    def test_create_collection_with_user_embeddings(
        self,
        client: VantageClient,
        account_params: dict,
        test_collection_id: str,
    ) -> None:
        """
        Tests creating an empty collection with user embeddings in given account.
        """
        # Given
        collection_id = test_collection_id
        collection_name = test_collection_id

        collection = UserProvidedEmbeddingsCollection(
            collection_id=collection_id,
            collection_name=collection_name,
            embeddings_dimension=1536,
        )

        # When
        collection = client.create_collection(
            account_id=account_params["id"],
            collection=collection,
        )

        # Then
        assert collection.collection_id == collection_id
        assert collection.collection_name == collection_name
        assert collection.collection_status == "Pending"
        assert collection.collection_state is None

    def test_upload_user_embeddings_to_a_collection(
        self,
        client: VantageClient,
        account_params: dict,
        test_collection_id: str,
        test_parquet_file_path: str,
    ) -> None:
        """
        Tests uploading user embeddings to a collection.
        """

        # Given
        collection_id = test_collection_id

        # When
        status = client.upload_documents_from_parquet_file(
            collection_id=collection_id,
            parquet_file_path=test_parquet_file_path,
            account_id=account_params["id"],
        )

        # Then
        assert status == 200

    def test_upload_user_embeddings_to_a_non_existing_collection(
        self,
        client: VantageClient,
        account_params: dict,
        test_collection_id: str,
        test_parquet_file_path: str,
    ) -> None:
        """
        Tests uploading user embeddings to a non-existing collection.
        """
        # Given
        collection_id = test_collection_id

        with pytest.raises(NotFoundException) as exception:
            client.upload_documents_from_parquet_file(
                collection_id=collection_id,
                parquet_file_path=test_parquet_file_path,
                account_id=account_params["id"],
            )

        # Then
        assert exception.type is NotFoundException

    def test_upload_non_existing_user_embeddings(
        self,
        client: VantageClient,
        account_params: dict,
        test_collection_id: str,
        random_string_generator: Callable,
    ) -> None:
        """
        Tests uploading non existing user embeddings file.
        """
        # Given
        collection_id = test_collection_id
        non_existing_file_path = random_string_generator(10)

        with pytest.raises(FileNotFoundError) as exception:
            client.upload_documents_from_parquet_file(
                collection_id=collection_id,
                parquet_file_path=non_existing_file_path,
                account_id=account_params["id"],
            )

        # Then
        assert exception.type is FileNotFoundError

    def test_upload_user_embeddings_with_wrong_file_size(
        self,
        client: VantageClient,
        account_params: dict,
        test_collection_id: str,
        test_parquet_file_path: str,
    ):
        """
        Tests uploading user embeddings with wrong file size specified.
        """
        # Given
        file_size = Path(test_parquet_file_path).stat().st_size + 1
        file = open(test_parquet_file_path, "rb")
        file_content = file.read()
        collection_id = test_collection_id

        # When
        with pytest.raises(VantageFileUploadError) as exception:
            client._upload_documents_from_bytes(
                collection_id=collection_id,
                content=file_content,
                file_size=file_size,
                batch_identifier="test-batch",
                account_id=account_params["id"],
            )

        # Then
        assert exception.type is VantageFileUploadError
        assert exception.value.args == ("Forbidden", 403)

    def test_create_vantage_managed_embeddings_collection(
        self,
        client: VantageClient,
        account_params: dict,
        test_collection_id: str,
    ) -> None:
        """
        Tests creating an empty collection with vantage managed
        embeddings in given account.
        """
        # Given
        collection_id = test_collection_id
        collection_name = test_collection_id

        collection = OpenAICollection(
            collection_id=collection_id,
            collection_name=collection_name,
            embeddings_dimension=1536,
            llm="text-embedding-ada-002",
            llm_secret="llmsecret",
        )

        # When
        collection = client.create_collection(
            account_id=account_params["id"],
            collection=collection,
        )

        # Then
        assert collection.collection_id == collection_id
        assert collection.collection_name == collection_name
        assert collection.collection_status == "Pending"
        assert collection.collection_state is None
        assert collection.llm_provider == "OpenAI"
        assert collection.llm == "text-embedding-ada-002"

    def test_list_collections(
        self,
        client: VantageClient,
        api_params: dict,
        account_params: dict,
    ) -> None:
        """
        Tests listing collections.
        This test will first create a number of collections,
        and then test if created collections are among listed collections.
        """
        # Given
        collection_ids = [
            "test-collection-1",
            "test-collection-2",
        ]

        for collection_id in collection_ids:
            create_temporary_collection(
                client=client,
                account_id=account_params["id"],
                collection_id=collection_id,
                collection_name=collection_id,
                user_provided_embeddings=True,
                embeddings_dimension=1536,
            )

        # When
        collections = client.list_collections()

        # Then
        created_collections = list(
            filter(
                lambda collection: collection.collection_name
                in collection_ids,
                collections,
            )
        )
        assert len(created_collections) == len(collection_ids)
        for collection in created_collections:
            assert collection.collection_name == collection.collection_id
            assert collection.collection_id in collection_ids
            assert collection.collection_name in collection_ids

    def test_get_collection(
        self,
        client: VantageClient,
        api_params: dict,
        account_params: dict,
        test_collection_id: str,
    ) -> None:
        """
        Tests if it can retrieve a single collection.
        """
        # Given
        collection_id = test_collection_id
        collection_name = test_collection_id

        collection = UserProvidedEmbeddingsCollection(
            collection_id=collection_id,
            embeddings_dimension=1536,
        )
        create_temporary_upe_collection(
            client=client,
            collection=collection,
            account_id=account_params["id"],
        )

        # When
        collection = client.get_collection(
            collection_id=collection_id,
            account_id=account_params["id"],
        )

        # Then
        assert collection.collection_id == collection_id
        assert collection.collection_name == collection_name
        assert collection.collection_status == "Online"
        assert collection.collection_state == "Active"

    def test_get_non_existing_collection(
        self,
        client: VantageClient,
        account_params: dict,
        test_collection_id: str,
    ) -> None:
        """
        Tests if fetching a non-existing collection will raise an exception.
        """
        # Given
        collection_id = test_collection_id

        # When
        with pytest.raises(NotFoundException) as exception:
            client.get_collection(
                collection_id=collection_id, account_id=account_params["id"]
            )

        # Then
        assert exception.type is NotFoundException

    def test_update_collection(
        self,
        client: VantageClient,
        api_params: dict,
        account_params: dict,
        test_collection_id: str,
    ) -> None:
        """
        Tests if an existing collection can be updated.
        """
        # Given
        collection_id = test_collection_id

        collection = UserProvidedEmbeddingsCollection(
            collection_id=collection_id,
            embeddings_dimension=1536,
        )
        create_temporary_upe_collection(
            client=client,
            collection=collection,
            account_id=account_params["id"],
        )

        updated_collection_name = "Updated Name"
        # Take a nap while API sets up a collection.
        time.sleep(3)

        # When
        collection = client.update_collection(
            collection_id=collection_id,
            account_id=account_params["id"],
            collection_name=updated_collection_name,
        )

        # Then
        assert collection.collection_name == updated_collection_name

    def test_update_non_existing_collection(
        self,
        client: VantageClient,
        account_params: dict,
        test_collection_id: str,
    ) -> None:
        """
        Tests if updating a non-existing collection will raise an exception.
        """
        # Given
        collection_id = test_collection_id
        updated_collection_name = "Updated Name"

        # When
        with pytest.raises(NotFoundException) as exception:
            client.update_collection(
                collection_id=collection_id,
                account_id=account_params["id"],
                collection_name=updated_collection_name,
            )

        # Then
        assert exception.type is NotFoundException

    def test_delete_collection(
        self,
        client: VantageClient,
        api_params: dict,
        account_params: dict,
        test_collection_id: str,
    ) -> None:
        """
        Tests if an existing collection can be deleted.
        """
        # Given
        collection_id = test_collection_id

        collection = UserProvidedEmbeddingsCollection(
            collection_id=collection_id,
            embeddings_dimension=1536,
        )
        create_temporary_upe_collection(
            client=client,
            collection=collection,
            account_id=account_params["id"],
        )

        # When
        client.delete_collection(
            collection_id=collection_id,
            account_id=account_params["id"],
        )

        # Then
        with pytest.raises(NotFoundException) as exception:
            client.get_collection(
                collection_id=collection_id,
                account_id=account_params["id"],
            )
        assert exception.type is NotFoundException

        listed_deleted_collection = list(
            filter(
                lambda col: col.collection_id == collection_id,
                client.list_collections(),
            )
        )
        assert len(listed_deleted_collection) == 0

    def test_delete_non_existing_collection(
        self,
        client: VantageClient,
        account_params: dict,
        test_collection_id: str,
    ) -> None:
        """
        Tests if deleting a non-existing collection will raise an exception.
        """
        # Given
        collection_id = test_collection_id

        # When
        with pytest.raises(NotFoundException) as exception:
            client.delete_collection(
                collection_id=collection_id, account_id=account_params["id"]
            )

        # Then
        assert exception.type is NotFoundException



import os

import pytest

from tests.integration_tests.configuration.loader import CONFIGURATION
from tests.integration_tests.configuration.mock_api import (
    get_request_stub_file_contents,
)
from tests.integration_tests.conftest import random_string


use_mock_api = CONFIGURATION["other"]["is_mock_api"]


@pytest.fixture
def updated_test_account_name(request) -> str:
    if not use_mock_api:
        return random_string(10)

    stub = get_request_stub_file_contents(request)
    return stub["response"]["jsonBody"]["account_name"]


@pytest.fixture
def non_existing_account_id(request) -> str:
    if not use_mock_api:
        return random_string(10)

    stub = get_request_stub_file_contents(request)

    return os.path.basename(stub["request"]["urlPath"])


@pytest.fixture(scope="module")
def account_params() -> dict:
    return CONFIGURATION["account"]



from typing import Callable

import pytest

from tests.integration_tests.conftest import disable_external_api_keys_tests
from vantage_sdk.client import VantageClient
from vantage_sdk.core.http.exceptions import (
    ForbiddenException,
    NotFoundException,
)


"""Integration tests for API keys endpoints."""


def _mask_secret(secret: str) -> str:
    if len(secret) < 5:
        return secret

    mask_char = "*"
    return f"{secret[0:5]}{(mask_char * (len(secret) - 5))}"


class TestApiKeys:
    def test_get_vantage_api_keys(
        self, client: VantageClient, account_params: dict
    ):
        """
        Tests fetching all vantage API keys present on a user account.
        """
        # When
        keys = client.get_vantage_api_keys(account_id=account_params["id"])

        # Then
        assert len(keys) == 1
        api_key = keys[0]
        assert api_key.vantage_api_key_obfuscated is not None
        assert api_key.account_id == account_params["id"]

    def test_get_vantage_api_keys_using_nonexisting_account(
        self,
        client: VantageClient,
        api_key_nonexisting_account_id: str,
    ):
        # When
        with pytest.raises(ForbiddenException) as exception:
            client.get_vantage_api_keys(
                account_id=api_key_nonexisting_account_id
            )

        # Then
        assert exception.type == ForbiddenException

    def test_get_vantage_api_key(
        self,
        client: VantageClient,
        account_params: dict,
        vantage_api_key_id: str,
    ):
        """
        Tests fetching single vantage API key present on a user account.
        """
        # When
        api_key = client.get_vantage_api_key(
            vantage_api_key_id=vantage_api_key_id,
            account_id=account_params["id"],
        )

        # Then
        assert api_key.account_id == account_params["id"]
        assert api_key.vantage_api_key_obfuscated is not None
        api_key.vantage_api_key_id == vantage_api_key_id

    def test_get_vantage_api_key_using_nonexisting_account(
        self,
        client: VantageClient,
        vantage_api_key_id: str,
        api_key_nonexisting_account_id: str,
    ):
        """
        Tests fetching a single Vantage API key using non-existing account ID.
        """
        # When
        with pytest.raises(ForbiddenException) as exception:
            client.get_vantage_api_key(
                vantage_api_key_id=vantage_api_key_id,
                account_id=api_key_nonexisting_account_id,
            )

        # Then
        assert exception.type == ForbiddenException

    def test_get_non_existing_vantage_api_key(
        self,
        client: VantageClient,
        account_params: dict,
        vantage_api_key_id: str,
        nonexisting_api_key_id: str,
    ):
        """
        Tests fetching a non-existing Vantage API key from a user account.
        """
        # When
        with pytest.raises(NotFoundException) as exception:
            client.get_vantage_api_key(
                vantage_api_key_id=nonexisting_api_key_id
            )

        # Then
        assert exception.type is NotFoundException

    @pytest.mark.skipif(
        reason=(
            "Cannot test with mock keys "
            "because of external API Key validation"
        ),
        condition=disable_external_api_keys_tests(),
    )
    def test_get_external_api_keys(
        self,
        client: VantageClient,
        account_params: dict,
        random_string_generator: Callable,
    ):
        """
        Tests fetching all external API keys from a users' account.
        """
        # Given
        llm_provider = "OpenAI"
        llm_secret = random_string_generator(10)
        given_key = client.create_external_key(
            llm_provider=llm_provider,
            llm_secret=llm_secret,
            account_id=account_params["id"],
        )
        # When
        keys = client.get_external_keys(account_id=account_params["id"])

        # Then
        assert len(keys) > 0
        api_keys = list(
            filter(
                lambda key: key.external_key_id == given_key.external_key_id,
                keys,
            )
        )
        assert len(api_keys) == 1
        api_key = api_keys[0]
        assert api_key.account_id == account_params["id"]
        assert api_key.external_key_id == given_key.external_key_id
        assert api_key.llm_provider == given_key.llm_provider
        assert api_key.llm_secret == _mask_secret(given_key.llm_secret)

    @pytest.mark.skipif(
        reason=(
            "Cannot test with mock keys "
            "because of external API Key validation"
        ),
        condition=disable_external_api_keys_tests(),
    )
    def test_get_external_api_key(
        self,
        client: VantageClient,
        account_params: dict,
        random_string_generator: Callable,
    ):
        """
        Tests fetching a single external API key from a users' account.
        """
        # Given
        llm_provider = "OpenAI"
        llm_secret = random_string_generator(10)
        given_key = client.create_external_key(
            llm_provider=llm_provider,
            llm_secret=llm_secret,
            account_id=account_params["id"],
        )

        # When
        api_key = client.get_external_key(
            account_id=account_params["id"],
            external_key_id=given_key.external_key_id,
        )

        # Then
        assert api_key.account_id == account_params["id"]
        assert api_key.external_key_id == given_key.external_key_id
        assert api_key.llm_provider == given_key.llm_provider
        assert api_key.llm_secret == _mask_secret(given_key.llm_secret)

    @pytest.mark.skipif(
        reason=(
            "Cannot test with mock keys "
            "because of external API Key validation"
        ),
        condition=disable_external_api_keys_tests(),
    )
    def test_get_non_existing_external_api_key(
        self,
        client: VantageClient,
        account_params: dict,
        nonexisting_external_api_key_id: str,
    ):
        """
        Tests fetching a non-existing external API key from a users' account.
        """
        # When
        with pytest.raises(NotFoundException) as exception:
            client.get_external_key(
                account_id=account_params["id"],
                external_key_id=nonexisting_external_api_key_id,
            )

        # Then
        assert exception.type is NotFoundException

    @pytest.mark.skipif(
        reason=(
            "Cannot test with mock keys "
            "because of external API Key validation"
        ),
        condition=disable_external_api_keys_tests(),
    )
    def test_create_external_api_key(
        self,
        client: VantageClient,
        account_params: dict,
        external_api_key_id: str,
        external_key_llm_secret: str,
    ):
        """
        Tests creating an external API key on a users' account.
        """
        # Given
        llm_provider = "OpenAI"
        llm_secret = external_key_llm_secret

        # When
        response = client.create_external_key(
            llm_provider=llm_provider,
            llm_secret=llm_secret,
            account_id=account_params["id"],
        )

        # Then
        assert response is not None
        assert response.account_id == account_params["id"]
        assert response.llm_provider == llm_provider
        assert response.llm_secret == llm_secret

    @pytest.mark.skipif(
        reason=(
            "Cannot test with mock keys "
            "because of external API Key validation"
        ),
        condition=disable_external_api_keys_tests(),
    )
    def test_update_external_api_key(
        self,
        client: VantageClient,
        account_params: dict,
        external_key_llm_secret: str,
        external_key_updated_llm_secret: str,
    ):
        """
        Tests updating an external API key present on a users' account.
        """
        # Given
        llm_provider = "OpenAI"
        llm_secret = external_key_llm_secret
        test_api_key = client.create_external_key(
            llm_provider=llm_provider,
            llm_secret=llm_secret,
            account_id=account_params["id"],
        )
        external_api_key_id = test_api_key.external_key_id

        # When
        client.update_external_key(
            external_key_id=external_api_key_id,
            llm_provider=llm_provider,
            llm_secret=external_key_updated_llm_secret,
            account_id=account_params["id"],
        )
        # When
        api_key = client.get_external_key(
            account_id=account_params["id"],
            external_key_id=external_api_key_id,
        )

        # Then
        assert api_key.account_id == account_params["id"]
        assert api_key.external_key_id == external_api_key_id
        assert api_key.llm_provider == llm_provider
        assert api_key.llm_secret == _mask_secret(
            external_key_updated_llm_secret
        )

    @pytest.mark.skipif(
        reason=(
            "Cannot test with mock keys "
            "because of external API Key validation"
        ),
        condition=disable_external_api_keys_tests(),
    )
    def test_update_non_existing_external_api_key(
        self,
        client: VantageClient,
        account_params: dict,
        nonexisting_external_api_key_id: str,
        external_key_llm_secret: str,
    ):
        """
        Tests updating a non-existing external API key
        on a users' account.
        """
        # When
        with pytest.raises(NotFoundException) as exception:
            client.update_external_key(
                external_key_id=nonexisting_external_api_key_id,
                llm_provider="OpenAI",
                llm_secret=external_key_llm_secret,
                account_id=account_params["id"],
            )

        # Then
        assert exception.type is NotFoundException

    @pytest.mark.skipif(
        reason=(
            "Cannot test with mock keys "
            "because of external API Key validation"
        ),
        condition=disable_external_api_keys_tests(),
    )
    def test_delete_external_api_key(
        self,
        client: VantageClient,
        account_params: dict,
        external_api_key_id: str,
    ):
        """
        Tests deleting an existing external API key on users' account.
        """
        # When
        api_key = client.delete_external_key(
            external_key_id=external_api_key_id,
            account_id=account_params["id"],
        )

        # Then
        assert api_key is None
        with pytest.raises(NotFoundException) as exception:
            client.get_external_key(
                external_key_id=external_api_key_id,
                account_id=account_params["id"],
            )
        assert exception.type is NotFoundException

    @pytest.mark.skipif(
        reason=(
            "Cannot test with mock keys "
            "because of external API Key validation"
        ),
        condition=disable_external_api_keys_tests(),
    )
    def test_delete_non_existing_external_api_key(
        self,
        client: VantageClient,
        account_params: dict,
        nonexisting_external_api_key_id: str,
    ):
        """
        Tests deleting a non-existing external API key on users' account.
        """
        # When
        with pytest.raises(NotFoundException) as exception:
            client.delete_external_key(
                external_key_id=nonexisting_external_api_key_id,
                account_id=account_params["id"],
            )

        # Then
        assert exception.type is NotFoundException



import os

import pytest

from tests.integration_tests.configuration.loader import CONFIGURATION
from tests.integration_tests.configuration.mock_api import (
    get_request_stub_file_contents,
)
from tests.integration_tests.conftest import random_string


use_mock_api = CONFIGURATION["other"]["is_mock_api"]


@pytest.fixture(scope="module")
def vantage_api_key() -> dict:
    vantage_api_key = CONFIGURATION["keys"].get("vantage_api_key")
    if vantage_api_key is None:
        pytest.skip("No Vantage API key available.")

    return vantage_api_key


@pytest.fixture(scope="module")
def vantage_api_key_id() -> dict:
    vantage_api_key_id = CONFIGURATION["keys"].get("vantage_api_key_id")
    if vantage_api_key_id is None:
        pytest.skip("No Vantage API key ID available.")

    return vantage_api_key_id


@pytest.fixture(scope="module")
def open_api_key() -> str:
    open_api_key = CONFIGURATION["keys"].get("open_api_key")
    if open_api_key is None:
        pytest.skip("No OpenAPI key available.")

    return open_api_key


@pytest.fixture(scope="module")
def external_api_key() -> str:
    external_api_key = CONFIGURATION["keys"].get("external_api_key")
    if external_api_key is None:
        pytest.skip("No external API key available.")

    return external_api_key


@pytest.fixture(scope="function")
def external_api_key_id(request) -> str:
    if not use_mock_api:
        external_api_key_id = CONFIGURATION["keys"].get("external_api_key_id")
        if external_api_key_id is None:
            pytest.skip("No external API key ID available.")

        return external_api_key_id

    stub = get_request_stub_file_contents(request)
    return os.path.basename(stub["request"]["urlPath"])


@pytest.fixture(scope="module")
def external_api_key_provider() -> str:
    external_api_key_provider = CONFIGURATION["keys"][
        "external_api_key_provider"
    ]
    if external_api_key_provider is None:
        pytest.skip("No external API key provider available.")

    return external_api_key_provider


@pytest.fixture(scope="function")
def api_key_nonexisting_account_id(request) -> str:
    if not use_mock_api:
        return random_string(10)

    stub = get_request_stub_file_contents(request)

    return stub["request"]["urlPath"].split("/")[3]


@pytest.fixture(scope="function")
def nonexisting_api_key_id(request) -> str:
    if not use_mock_api:
        return random_string(10)

    stub = get_request_stub_file_contents(request)

    return os.path.basename(stub["request"]["urlPath"])


@pytest.fixture(scope="function")
def nonexisting_external_api_key_id(request) -> str:
    if not use_mock_api:
        return random_string(10)

    stub = get_request_stub_file_contents(request)

    return os.path.basename(stub["request"]["urlPath"])


@pytest.fixture(scope="function")
def external_key_llm_secret(request) -> str:
    if not use_mock_api:
        return random_string(10)

    stub = get_request_stub_file_contents(request)

    try:
        return os.path.basename(stub["response"]["jsonBody"]["llm_secret"])
    except KeyError:
        return CONFIGURATION["keys"]["llm_secret"]


@pytest.fixture(scope="function")
def external_key_updated_llm_secret(request) -> str:
    if not use_mock_api:
        return random_string(10)

    stub = get_request_stub_file_contents(request)

    return os.path.basename(stub["response"]["jsonBody"]["llm_secret"])



"""
Models for the Collection API.
"""

from typing import List, Optional

from pydantic import (
    BaseModel,
    StrictBool,
    StrictInt,
    StrictStr,
    model_validator,
)

from vantage_sdk.model.keys import LLMProvider, SecondaryExternalAccount


class Collection(BaseModel):
    """
    A collection of documents.

    Attributes
    ----------
    collection_id: str
        ID of the collection.
    embeddings_dimension: int
        Dimension of embeddings in the collection documents.
    user_provided_embeddings: bool
        If the collection uses user-provided embeddings.
    collection_name: Optional[StrictStr]
        Name of the collection.
    collection_state: Optional[StrictStr]
        Current state of the collection.
    collection_status: Optional[StrictStr]
        Current status of the collection.
    collection_created_time: Optional[StrictStr]
        When the collection was created.
    collection_preview_url_pattern: Optional[StrictStr]
        Preview URL pattern of the collection.
    """

    collection_id: StrictStr
    embeddings_dimension: StrictInt
    user_provided_embeddings: StrictBool
    collection_name: Optional[StrictStr] = None
    collection_state: Optional[StrictStr] = None
    collection_status: Optional[StrictStr] = None
    collection_created_time: Optional[StrictStr] = None
    collection_preview_url_pattern: Optional[StrictStr] = None

    @model_validator(mode="before")
    def set_default_collection_name(cls, values):
        if (
            "collection_name" not in values
            or values["collection_name"] is None
        ):
            collection_id = values.get("collection_id")
            if collection_id:
                values["collection_name"] = f"Collection [{collection_id}]"
        return values


class UserProvidedEmbeddingsCollection(Collection):
    """
    A collection of documents with user-provided embeddings.

    Attributes
    ----------
    user_provided_embeddings : StrictBool
        Indicates that the embeddings in this collection are provided by the user.

    """

    user_provided_embeddings: StrictBool = True


class VantageManagedEmbeddingsCollection(Collection):
    """
    A collection of documents with Vantage-managed embeddings.

    Attributes
    ----------
    user_provided_embeddings : StrictBool
        Indicates that the embeddings in this collection are managed by Vantage.
    llm_provider : StrictStr
        The provider of the embedding model used to generate embeddings.
    llm_secret : Optional[StrictStr], optional
        The secret key used to authenticate with the model provider.
    external_key_id : Optional[StrictStr], optional
        The external key ID used to authenticate with the model provider.
    """

    user_provided_embeddings: StrictBool = False
    llm_provider: StrictStr
    llm_secret: Optional[StrictStr] = None
    external_key_id: Optional[StrictStr] = None

    @model_validator(mode="after")
    def check_extrnal_authentication_method_exists(cls, values):
        if not values.llm_secret and not values.external_key_id:
            raise ValueError(
                "Please provide either `llm_secret` or `external_key_id` to authenticate with the LLM provider."
            )


class OpenAICollection(VantageManagedEmbeddingsCollection):
    """
    Vantage-managed embeddings collection which uses OpenAI as LLM provider.

    Attributes
    ----------
    llm_provider : StrictStr
        The provider of the embedding model, set to OpenAI.
    llm : StrictStr
        The specific embedding model used from OpenAI.
    secondary_external_accounts : Optional[List[SecondaryExternalAccount]], optional
        A list of secondary external accounts associated with the collection.
    """

    llm_provider: StrictStr = LLMProvider.OpenAI.value
    llm: StrictStr
    secondary_external_accounts: Optional[
        List[SecondaryExternalAccount]
    ] = None


class HuggingFaceCollection(VantageManagedEmbeddingsCollection):
    """
    Vantage-managed embeddings collection which uses HuggingFace as LLM provider.

    Attributes
    ----------
    llm_provider : StrictStr
        The provider of the embedding model, set to HuggingFace.
    external_url : StrictStr
        The external URL for the HuggingFace model.
    """

    llm_provider: StrictStr = LLMProvider.HuggingFace.value
    external_url: StrictStr = None


class CollectionUploadURL(BaseModel):
    """
    Represents the upload URL details for a collection.

    Attributes
    ----------
    collection_id : Optional[StrictStr], optional
        The unique identifier for the collection.
    customer_batch_identifier : Optional[StrictStr], optional
        The identifier for the customer batch.
    upload_url_type : Optional[StrictStr], optional
        The type of the upload URL.
    upload_url : Optional[StrictStr], optional
        The URL used for uploading to the collection.
    """

    collection_id: Optional[StrictStr] = None
    customer_batch_identifier: Optional[StrictStr] = None
    upload_url_type: Optional[StrictStr] = None
    upload_url: Optional[StrictStr] = None



# coding: utf-8

"""
    Vantage API

    This is a the API to interact with Vantage Discovery, the amazing Semantic Search Platform in the world.  We enable developers to build magical discovery experiences into their products and websites.  Some useful links: - [TODO: Semantic Search Guide: What Is It And Why Does It Matter?](https://www.bloomreach.com/en/blog/2019/semantic-search-explained-in-5-minutes)

    The version of the OpenAPI document: v1.1.2
    Contact: devrel@vantagediscovery.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import Any, ClassVar, Dict, List, Optional, Union

from pydantic import (
    BaseModel,
    StrictFloat,
    StrictInt,
    StrictStr,
    field_validator,
)

from vantage_sdk.core.http.models.weighted_field_values import (
    WeightedFieldValues,
)


try:
    from typing import Self
except ImportError:
    from typing_extensions import Self


class SearchOptionsFieldValueWeighting(BaseModel):
    """
    SearchOptionsFieldValueWeighting
    """  # noqa: E501

    query_key_word_max_overall_weight: Optional[
        Union[StrictFloat, StrictInt]
    ] = None
    query_key_word_weighting_mode: Optional[StrictStr] = None
    weighted_field_values: Optional[List[WeightedFieldValues]] = None
    __properties: ClassVar[List[str]] = [
        "query_key_word_max_overall_weight",
        "query_key_word_weighting_mode",
        "weighted_field_values",
    ]

    @field_validator('query_key_word_weighting_mode')
    def query_key_word_weighting_mode_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in ('none', 'uniform', 'weighted'):
            raise ValueError(
                "must be one of enum values ('none', 'uniform', 'weighted')"
            )
        return value

    model_config = {
        "populate_by_name": True,
        "validate_assignment": True,
        "protected_namespaces": (),
    }

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of SearchOptionsFieldValueWeighting from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        _dict = self.model_dump(
            by_alias=True,
            exclude={},
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in weighted_field_values (list)
        _items = []
        if self.weighted_field_values:
            for _item in self.weighted_field_values:
                if _item:
                    _items.append(_item.to_dict())
            _dict['weighted_field_values'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: Dict) -> Self:
        """Create an instance of SearchOptionsFieldValueWeighting from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "query_key_word_max_overall_weight": obj.get(
                    "query_key_word_max_overall_weight"
                ),
                "query_key_word_weighting_mode": obj.get(
                    "query_key_word_weighting_mode"
                ),
                "weighted_field_values": [
                    WeightedFieldValues.from_dict(_item)
                    for _item in obj.get("weighted_field_values")
                ]
                if obj.get("weighted_field_values") is not None
                else None,
            }
        )
        return _obj



# flake8: noqa

# import apis into api package
from vantage_sdk.core.http.api.account_management_api import (
    AccountManagementApi,
)
from vantage_sdk.core.http.api.collection_management_api import (
    CollectionManagementApi,
)
from vantage_sdk.core.http.api.documents_api import DocumentsApi
from vantage_sdk.core.http.api.external_keys_api import ExternalKeysApi
from vantage_sdk.core.http.api.search_api import SearchApi
from vantage_sdk.core.http.api.shopping_assistant_api import (
    ShoppingAssistantApi,
)
from vantage_sdk.core.http.api.vantage_api_keys_api import VantageAPIKeysApi
from vantage_sdk.core.http.api.vantage_vibe_api import VantageVibeApi



"""
Models for the Documents API.
"""

from typing import Dict, List, Optional, Union
from uuid import uuid4

from pydantic import (
    BaseModel,
    Field,
    StrictBool,
    StrictFloat,
    StrictInt,
    StrictStr,
    model_validator,
)

from vantage_sdk.config import METADATA_ORDERED_PREFIX, METADATA_PREFIX


class MetadataItem(BaseModel):
    """
    Represents a metadata item associated with a document.

    Attributes
    ----------
    key : StrictStr
        The key for the metadata item.
    value : Union[StrictStr, StrictInt, StrictFloat, StrictBool]
        The value for the metadata item.
    """

    key: StrictStr
    value: Union[StrictStr, StrictInt, StrictFloat, StrictBool]

    @model_validator(mode="before")
    def add_prefix(cls, values: Dict):
        key, value = values.get("key"), values.get("value")
        if isinstance(value, float):
            prefix = METADATA_ORDERED_PREFIX
        else:
            prefix = METADATA_PREFIX

        if key:
            values["key"] = prefix + key
        return values


class VantageDocument(BaseModel):
    """
    Vantage document class.

    Attributes
    ----------
    text : StrictStr
        The text content of the document.
    id : Optional[StrictStr], optional
        The unique identifier for the document, generated by default if not provided.
    metadata : Optional[List[MetadataItem]], optional
        A list of metadata items associated with the document.
    """

    text: StrictStr
    id: Optional[StrictStr] = Field(default_factory=lambda: uuid4().hex)
    metadata: Optional[List[MetadataItem]] = None

    def to_vantage_dict(self):
        vantage_dict = {
            "id": self.id,
            "text": self.text,
        }

        if self.metadata:
            for metadata_item in self.metadata:
                vantage_dict[metadata_item.key] = metadata_item.value

        return vantage_dict


class VantageManagedEmbeddingsDocument(VantageDocument):
    """Document class for Vantage-managed embeddings collections"""


class UserProvidedEmbeddingsDocument(VantageDocument):
    """Document class for User-provided embeddings collections"""

    embeddings: List[StrictFloat]

    def to_vantage_dict(self):
        vantage_dict = super().to_vantage_dict()
        vantage_dict["embeddings"] = self.embeddings

        return vantage_dict



import pytest

from tests.integration_tests.configuration.loader import CONFIGURATION


@pytest.fixture(scope="module")
def embedding_search_test_collection_id() -> str:
    embedding_search_test_collection_id = CONFIGURATION["collection"][
        "embedding_search_test_collection_id"
    ]

    if embedding_search_test_collection_id is None:
        pytest.skip("No embedding search test collection available.")

    return embedding_search_test_collection_id


@pytest.fixture(scope="module")
def semantic_search_test_collection_id() -> str:
    semantic_search_test_collection_id = CONFIGURATION["collection"][
        "semantic_search_test_collection_id"
    ]

    if semantic_search_test_collection_id is None:
        pytest.skip("No semantic search test collection available.")

    return semantic_search_test_collection_id


@pytest.fixture(scope="module")
def more_like_this_test_collection_id() -> str:
    mlt_search_test_collection_id = CONFIGURATION["collection"].get(
        "more_like_this_collection"
    )

    if mlt_search_test_collection_id is None:
        pytest.skip("No more like this search test collection available.")

    return mlt_search_test_collection_id


@pytest.fixture(scope="module")
def embedding_search_test_collection_id_for_setup() -> str:
    embedding_search_test_collection_id = CONFIGURATION["collection"].get(
        "embedding_search_test_collection_id"
    )

    if embedding_search_test_collection_id is None:
        pytest.skip("No embedding search test collection available.")

    return embedding_search_test_collection_id


@pytest.fixture(scope="module")
def semantic_search_test_collection_id_for_setup() -> str:
    semantic_search_test_collection_id = CONFIGURATION["collection"].get(
        "semantic_search_test_collection_id"
    )

    if semantic_search_test_collection_id is None:
        pytest.skip("No embedding search test collection available.")

    return semantic_search_test_collection_id



import os

import pytest

from tests.integration_tests.configuration.loader import CONFIGURATION
from tests.integration_tests.configuration.mock_api import (
    get_mock_request_for,
    get_request_stub_file_contents,
)


use_mock_api = CONFIGURATION["other"]["is_mock_api"]


def _get_collection_id_for_mock_api(request) -> str:
    """Get collection names directly from wiremock stub files."""
    module_name = os.path.basename(request.path).replace(".py", "")
    test_name = request.node.name

    if test_name in (
        "test_documents_upload_from_parquet_file",
        "test_documents_upload_from_jsonl_file",
    ):
        stub = get_mock_request_for(
            module_name="test_documents",
            test_name="test_documents_upload_from_parquet_file-upload_url",
        )
    else:
        stub = get_request_stub_file_contents(request)

    if module_name == "test_search":
        return stub["request"]["urlPath"].split("/")[4]

    if test_name in (
        "test_create_collection_with_user_embeddings",
        "test_create_vantage_managed_embeddings_collection",
    ):
        return stub["response"]["jsonBody"]["collection_id"]

    urlPart = None
    if "url" in stub["request"]:
        urlPart = stub["request"]["url"]
    else:
        urlPart = stub["request"]["urlPath"]

    return urlPart.split("/")[5]


def _get_collection_id_for_real_api(request) -> str:
    collection = CONFIGURATION["collection"]
    test_name = request.node.name

    if test_name in (
        "test_upload_user_embeddings_to_a_non_existing_collection",
        "test_get_non_existing_collection",
        "test_update_non_existing_collection",
        "test_delete_non_existing_collection",
        "test_embedding_search_on_non_existing_collection",
        "test_semantic_search_on_non_existing_collection",
    ):
        return collection["non_existing_collection_id"]

    if test_name == "test_update_collection":
        return collection["collection_to_update_id"]

    if test_name == "test_delete_collection":
        return collection["collection_to_delete_id"]

    if test_name == "test_if_embedding_search_returns_result":
        return collection["embedding_search_test_collection_id"]

    if test_name == "test_if_semantic_search_returns_result":
        return collection["semantic_search_test_collection_id"]

    return collection["collection_id"]


@pytest.fixture(scope="module")
def collection_params() -> dict:
    return CONFIGURATION["collection"]


@pytest.fixture(scope="function")
def test_collection_id(request) -> str:
    if not use_mock_api:
        return CONFIGURATION["collection"]["id"]

    return _get_collection_id_for_mock_api(request)



from typing import Optional, Union

from tests.integration_tests.configuration.loader import CONFIGURATION
from vantage_sdk.client import VantageClient
from vantage_sdk.model.collection import (
    HuggingFaceCollection,
    OpenAICollection,
    UserProvidedEmbeddingsCollection,
)


def create_temporary_upe_collection(
    client: VantageClient,
    collection: UserProvidedEmbeddingsCollection,
    account_id: Optional[str] = None,
) -> Union[
    UserProvidedEmbeddingsCollection,
    OpenAICollection,
    HuggingFaceCollection,
]:
    if CONFIGURATION["other"]["is_mock_api"]:
        return collection

    return client.create_collection(
        account_id=account_id,
        collection=collection,
    )


def create_temporary_collection(
    client: VantageClient,
    collection_id: str,
    collection_name: str,
    user_provided_embeddings: bool,
    embeddings_dimension: int,
    account_id: Optional[str] = None,
) -> Optional[
    Union[
        UserProvidedEmbeddingsCollection,
        OpenAICollection,
        HuggingFaceCollection,
    ]
]:
    if CONFIGURATION["other"]["is_mock_api"]:
        return None

    return client.create_collection(
        account_id=account_id,
        collection_id=collection_id,
        collection_name=collection_name,
        user_provided_embeddings=user_provided_embeddings,
        embeddings_dimension=embeddings_dimension,
    )



from enum import Enum
from typing import Any, Optional

from pydantic import BaseModel


class CollectionType(Enum):
    """Collection type"""

    USER_PROVIDED_EMBEDDINGS = "UPE"
    OPEN_AI = "OpenAI"
    HUGGING_FACE = "HuggingFace"


class ErrorMessage(BaseModel):
    """
    Validation error message.

    Arguments
    ---------
    field_name: Optional[str]
        Name of field for which is erroneous.
    error_message: str
        Message describing error details.
    """

    field_name: Optional[str]
    error_message: str


class ValidationError(BaseModel):
    """
    Validation error for a single document.

    Arguments
    ---------
    document_id: Optional[str]
        Unique id of a document.
    line_number: Optional[int]
        A line/row where document is located in file.
    errors: list[ErrorMessage]
        Validation error messages.
    """

    document_id: Optional[str]
    line_number: Optional[int]
    errors: list[ErrorMessage]

    def to_dict(self) -> dict[str, Any]:
        return {
            "document_id": self.document_id,
            "line_number": self.line_number,
            "errors": [error.__dict__ for error in self.errors],
        }



from typing import List

import pytest

from vantage_sdk.model.document import (
    UserProvidedEmbeddingsDocument,
    VantageManagedEmbeddingsDocument,
)


@pytest.fixture(scope="module")
def jsonl_documents_path() -> str:
    return "tests/data/small_documents.jsonl"


@pytest.fixture(scope="module")
def parquet_file_path() -> str:
    return "tests/data/hello_world.parquet"


@pytest.fixture(scope="module")
def test_parquet_file_path() -> str:
    return "tests/data/hello_world.parquet"


@pytest.fixture(scope="module")
def vantage_upe_documents() -> List[UserProvidedEmbeddingsDocument]:
    ids = [
        "1",
        "2",
        "3",
        "4",
    ]

    texts = [
        "First text",
        "Second text",
        "Third text",
        "Fourth text",
    ]

    embeddings = [
        [0.123, 0.234, 0.345],
        [0.456, 0.567, 0.678],
        [0.789, 0.891, 0.912],
        [0.257, 0.389, 0.468],
    ]
    documents = [
        UserProvidedEmbeddingsDocument(text=text, id=id, embeddings=emb)
        for id, text, emb in zip(ids, texts, embeddings)
    ]

    return documents


@pytest.fixture(scope="module")
def vantage_vme_documents() -> List[VantageManagedEmbeddingsDocument]:
    ids = [
        "1",
        "2",
        "3",
        "4",
    ]

    texts = [
        "First text",
        "Second text",
        "Third text",
        "Fourth text",
    ]

    documents = [
        VantageManagedEmbeddingsDocument(text=text, id=id)
        for id, text in zip(ids, texts)
    ]

    return documents



import os


ABS_PATH = os.path.dirname(os.path.abspath(__file__))
PROJECT_DIR = os.path.abspath(os.path.join(ABS_PATH, os.pardir, os.pardir))
DISABLE_EXTERNAL_API_KEYS_TESTS = True
MOCK_DOTENV_FILE = os.path.join(PROJECT_DIR, "integration_tests", "mock-env")


def _load_env() -> None:
    dotenv_path = os.path.join(PROJECT_DIR, "integration_tests", ".env")
    if os.path.exists(dotenv_path):
        from dotenv import load_dotenv

        load_dotenv(dotenv_path)
    elif os.path.exists(MOCK_DOTENV_FILE):
        from dotenv import load_dotenv

        load_dotenv(MOCK_DOTENV_FILE)
    else:
        raise FileNotFoundError("Could not find valid environment file.")


def should_enable_external_api_key_tests(is_mock_api: bool) -> bool:
    enable_tests_env = (
        True
        if os.getenv("DISABLE_EXTERNAL_API_KEYS_TESTS", "true") == "true"
        else False
    )
    return enable_tests_env or is_mock_api


def _load_configuration() -> None:
    configuration = {
        "api": {
            "client_id": os.getenv("VANTAGE_CLIENT_ID"),
            "client_secret": os.getenv("VANTAGE_CLIENT_SECRET"),
            "auth_host": os.getenv("VANTAGE_AUTH_HOST"),
            "api_host": os.getenv("VANTAGE_API_HOST"),
            "is_mock": (
                True if os.getenv("USE_MOCK_API", "false") == "true" else False
            ),
        },
        "account": {
            "id": os.getenv("TEST_ACCOUNT_ID"),
            "name": os.getenv("TEST_ACCOUNT_NAME"),
        },
        "collection": {
            "embedding_search_test_collection_id": os.getenv(
                "VANTAGE_EMBEDDING_SEARCH_TEST_COLLECTION_ID"
            ),
            "semantic_search_test_collection_id": os.getenv(
                "VANTAGE_SEMANTIC_SEARCH_TEST_COLLECTION_ID"
            ),
            "more_like_this_collection": os.getenv(
                "VANTAGE_MORE_LIKE_THIS_SEARCH_COLLECTION_ID"
            ),
            "non_existing_collection_id": os.getenv(
                "NON_EXISTING_COLLECTION_ID"
            ),
            "collection_to_update_id": os.getenv("COLLECTION_TO_UPDATE_ID"),
            "collection_to_delete_id": os.getenv("COLLECTION_TO_DELETE_ID"),
            "collection_id": os.getenv("GENERAL_COLLECTION_ID"),
            "collection_name": os.getenv("GENERAL_COLLECTION_NAME"),
        },
        "keys": {
            "vantage_api_key": os.getenv("VANTAGE_API_KEY"),
            "vantage_api_key_id": os.getenv("VANTAGE_API_KEY_ID"),
            "open_api_key": os.getenv("OPEN_API_KEY"),
            "open_api_key_id": os.getenv("OPEN_API_KEY_ID"),
            "external_api_key": os.getenv("EXTERNAL_API_KEY"),
            "external_api_key_id": os.getenv("EXTERNAL_API_KEY_ID"),
            "external_api_key_provider": os.getenv(
                "EXTERNAL_API_KEY_PROVIDER"
            ),
            "llm_secret": os.getenv("LLM_SECRET", None),
        },
        "auth": {
            "auth_method": os.getenv(
                "VANTAGE_AUTH_METHOD",
                "client_credentials",
            ),
            "jwt_token": os.getenv("VANTAGE_API_JWT_TOKEN", None),
        },
        "other": {
            "is_mock_api": (
                True if os.getenv("USE_MOCK_API", "false") == "true" else False
            )
        },
    }
    configuration["other"][
        "enable_external_api_tests"
    ] = should_enable_external_api_key_tests(
        configuration["other"]["is_mock_api"]
    )
    return configuration


_load_env()
CONFIGURATION = _load_configuration()



DEFAULT_API_HOST = "https://api.vanta.ge"
API_HOST_VERSION = "v1"

DEFAULT_AUTH_HOST = "https://auth.vanta.ge"
AUTH_ENDPOINT = "/oauth/token"

DEFAULT_ENCODING = "utf-8"

METADATA_PREFIX = "meta_"
METADATA_ORDERED_PREFIX = "meta_ordered_"



#!/usr/bin/env python

import pytest

from vantage_sdk.client import VantageClient
from vantage_sdk.core.http.exceptions import (
    BadRequestException,
    UnauthorizedException,
)
from vantage_sdk.model.search import (
    Facet,
    FacetType,
    Filter,
    MoreLikeTheseItem,
    VantageVibeImageBase64,
    VantageVibeImageUrl,
)


"""Integration tests for search endpoints"""


class TestSearch:
    def test_if_embedding_search_returns_result(
        self,
        client: VantageClient,
        account_params: dict,
        test_collection_id: str,
    ):
        """
        Tests if embedding search will return correct result.
        """
        # Given
        collection_id = test_collection_id
        search_embedding = [1, 1, 1, 1, 1]

        # When
        result = client.embedding_search(
            embedding=search_embedding,
            collection_id=collection_id,
            account_id=account_params["id"],
        )

        # Then
        assert result.status == 200
        assert len(result.results) == 10

    def test_embedding_search_on_non_existing_collection(
        self,
        client: VantageClient,
        account_params: dict,
        test_collection_id: str,
    ):
        """
        Tests if searching a non-existing collection will raise an exception.
        """
        # Given
        collection_id = test_collection_id
        search_embedding = [1, 1, 1, 1, 1]

        # When
        with pytest.raises(UnauthorizedException) as exception:
            client.embedding_search(
                embedding=search_embedding,
                collection_id=collection_id,
                account_id=account_params["id"],
            )

        # Then
        assert exception.type is UnauthorizedException

    def test_embedding_search_with_invalid_embedding(
        self,
        client: VantageClient,
        account_params: dict,
        test_collection_id: str,
    ):
        """
        Tests if searching using an empty embedding will raise an exception.
        """
        # Given
        collection_id = test_collection_id
        search_embedding = []

        # When
        with pytest.raises(BadRequestException) as exception:
            client.embedding_search(
                embedding=search_embedding,
                collection_id=collection_id,
                account_id=account_params["id"],
            )

        # Then
        assert exception.type is BadRequestException

    def test_embedding_search_with_invalid_accuracy(
        self,
        client: VantageClient,
        account_params: dict,
        test_collection_id: str,
    ):
        """
        Tests if searching using invalid accuracy parameter
        will raise an exception.
        """
        # Given
        collection_id = test_collection_id
        search_embedding = [1, 1, 1, 1, 1]
        accuracy = 800

        # When
        with pytest.raises(BadRequestException) as exception:
            client.embedding_search(
                embedding=search_embedding,
                collection_id=collection_id,
                accuracy=accuracy,
                account_id=account_params["id"],
            )

        # Then
        assert exception.type is BadRequestException

    def test_if_semantic_search_returns_result(
        self,
        client: VantageClient,
        account_params: dict,
        test_collection_id: str,
    ):
        """
        Tests if semantic search will return correct result.
        """
        # Given
        collection_id = test_collection_id
        accuracy = 0.2
        search_text = "short legs and long body"

        # When
        result = client.semantic_search(
            text=search_text,
            collection_id=collection_id,
            accuracy=accuracy,
            account_id=account_params["id"],
        )

        # Then
        assert result.status == 200
        assert len(result.results) == 10

    def test_semantic_search_on_non_existing_collection(
        self,
        client: VantageClient,
        account_params: dict,
        test_collection_id: str,
    ):
        """
        Tests if performing a semantic search on an non-existing collection
        will raise an exception.
        """
        # Given
        collection_id = test_collection_id
        search_text = "Test search"

        # When
        with pytest.raises(UnauthorizedException) as exception:
            client.semantic_search(
                text=search_text,
                collection_id=collection_id,
                account_id=account_params["id"],
            )

        # Then
        assert exception.type is UnauthorizedException

    def test_semantic_search_with_invalid_accuracy(
        self,
        client: VantageClient,
        account_params: dict,
        test_collection_id: str,
    ):
        """
        Tests if performing a semantic search with an invalid
        accuracy parameter will raise an exception.
        """

        # Given
        collection_id = test_collection_id
        search_text = "Test search"
        accuracy = 800

        # When
        with pytest.raises(BadRequestException) as exception:
            client.semantic_search(
                text=search_text,
                collection_id=collection_id,
                accuracy=accuracy,
                account_id=account_params["id"],
            )

        # Then
        assert exception.type is BadRequestException

    def test_more_like_this_search(
        self,
        client: VantageClient,
        account_params: dict,
        test_collection_id: str,
    ) -> None:
        """
        Tests if MoreLikeThis search will return correct result.
        """
        # Given
        collection_id = test_collection_id
        expected_results = {
            "en_0370917": {"score": 0.907},
            'en_0127807': {"score": 0.904},
            'en_0772990': {"score": 0.904},
        }

        # When
        response = client.more_like_this_search(
            collection_id=collection_id,
            document_id="en_0530926",
            account_id=account_params["id"],
        )

        # Then
        assert response is not None
        assert response.status == 200
        results = response.results
        assert len(results) == len(expected_results)
        for result in results:
            assert result.id in expected_results.keys()
            assert expected_results[result.id]["score"] == round(
                result.score, 3
            )

    def test_more_like_these_search(
        self,
        client: VantageClient,
        account_params: dict,
        test_collection_id: str,
    ) -> None:
        """
        Tests if MoreLikeThese search will return correct result.
        """
        # Given
        collection_id = test_collection_id
        more_like_these = [
            MoreLikeTheseItem(
                weight=1.0,
                query_text="some text",
            ),
            MoreLikeTheseItem(
                weight=1.0,
                query_text="other text",
            ),
        ]
        expected_results = {
            "en_0022659": {"score": 0.891},
            "en_0375881": {"score": 0.891},
            "en_0266579": {"score": 0.891},
            "en_0218966": {"score": 0.891},
            "en_0622322": {"score": 0.89},
        }

        response = client.more_like_these_search(
            collection_id=collection_id,
            account_id=account_params["id"],
            more_like_these=more_like_these,
        )

        # Then
        assert response is not None
        assert response.status == 200
        results = response.results
        assert len(results) == len(expected_results)
        for result in results:
            assert result.id in expected_results.keys()
            assert expected_results[result.id]["score"] == round(
                result.score, 3
            )

    # region Variants

    def test_semantic_search_with_variant_filter(
        self,
        client: VantageClient,
        account_params: dict,
        test_collection_id: str,
    ):
        """
        Tests if semantic search will return correct result using variant filter option.
        """
        # Given
        collection_id = test_collection_id
        search_text = "Test search"

        filter = Filter(
            variant_filter="(color:\"black\" OR color:\"brown\")",
        )

        # When
        result = client.semantic_search(
            text=search_text,
            collection_id=collection_id,
            filter=filter,
            account_id=account_params["id"],
        )

        # Then
        assert result.status == 200
        assert len(result.results) == 3
        for r in result.results:
            assert "variants" in r.model_dump().keys()
        for r in result.results:
            assert "variants_full_list" in r.model_dump().keys()

    def test_embedding_search_with_variant_filter(
        self,
        client: VantageClient,
        account_params: dict,
        test_collection_id: str,
    ):
        """
        Tests if embedding search will return correct result using variant filter option.
        """
        # Given
        collection_id = test_collection_id
        embedding = [1, 1, 1, 1, 1]

        filter = Filter(
            variant_filter="(color:\"black\" OR color:\"brown\")",
        )

        # When
        result = client.embedding_search(
            embedding=embedding,
            collection_id=collection_id,
            filter=filter,
            account_id=account_params["id"],
        )

        # Then
        assert result.status == 200
        assert len(result.results) == 3
        for r in result.results:
            assert "variants" in r.model_dump().keys()
        for r in result.results:
            assert "variants_full_list" in r.model_dump().keys()

    def test_more_like_this_search_with_variant_filter(
        self,
        client: VantageClient,
        account_params: dict,
        test_collection_id: str,
    ):
        """
        Tests if MoreLikeThis search will return correct result using variant filter option.
        """
        # Given
        collection_id = test_collection_id
        document_id = "en_0530926"

        filter = Filter(
            variant_filter="(color:\"black\" OR color:\"brown\")",
        )

        # When
        result = client.more_like_this_search(
            document_id=document_id,
            collection_id=collection_id,
            filter=filter,
            account_id=account_params["id"],
        )

        # Then
        assert result.status == 200
        assert len(result.results) == 3
        for r in result.results:
            assert "variants" in r.model_dump().keys()
        for r in result.results:
            assert "variants_full_list" in r.model_dump().keys()

    def test_more_like_these_search_with_variant_filter(
        self,
        client: VantageClient,
        account_params: dict,
        test_collection_id: str,
    ):
        """
        Tests if MoreLikeThese search will return correct result using variant filter option.
        """
        # Given
        collection_id = test_collection_id
        these = [
            MoreLikeTheseItem(
                weight=1.0,
                query_text="some text",
            ),
            MoreLikeTheseItem(
                weight=1.0,
                query_text="other text",
            ),
        ]

        filter = Filter(
            variant_filter="(color:\"black\" OR color:\"brown\")",
        )

        # When
        result = client.more_like_these_search(
            more_like_these=these,
            collection_id=collection_id,
            filter=filter,
            account_id=account_params["id"],
        )

        # Then
        assert result.status == 200
        assert len(result.results) == 3
        for r in result.results:
            assert "variants" in r.model_dump().keys()
        for r in result.results:
            assert "variants_full_list" in r.model_dump().keys()

    # endregion

    # region Facets

    def test_semantic_search_with_facets(
        self,
        client: VantageClient,
        account_params: dict,
        test_collection_id: str,
    ):
        """
        Tests if semantic search will return correct result with facets.
        """
        # Given
        collection_id = test_collection_id
        search_text = "Test search"

        facets = [
            Facet(
                name="color",
                type=FacetType.COUNT,
            ),
            Facet(
                name="size",
                type=FacetType.COUNT,
                values=["sm", "md"],
            ),
        ]

        # When
        result = client.semantic_search(
            text=search_text,
            collection_id=collection_id,
            facets=facets,
            account_id=account_params["id"],
        )

        # Then
        assert result.status == 200
        assert len(result.results) == 3
        assert len(result.facets) == len(facets)

    def test_embedding_search_with_facets(
        self,
        client: VantageClient,
        account_params: dict,
        test_collection_id: str,
    ):
        """
        Tests if embedding search will return correct result with facets.
        """
        # Given
        collection_id = test_collection_id
        embedding = [1, 1, 1, 1, 1]

        facets = [
            Facet(
                name="color",
                type=FacetType.COUNT,
            ),
            Facet(
                name="size",
                type=FacetType.COUNT,
                values=["sm", "md"],
            ),
        ]

        # When
        result = client.embedding_search(
            embedding=embedding,
            collection_id=collection_id,
            facets=facets,
            account_id=account_params["id"],
        )

        # Then
        assert result.status == 200
        assert len(result.results) == 3
        assert len(result.facets) == len(facets)

    def test_more_like_this_search_with_facets(
        self,
        client: VantageClient,
        account_params: dict,
        test_collection_id: str,
    ):
        """
        Tests if MoreLikeThis search will return correct result with facets.
        """
        # Given
        collection_id = test_collection_id
        document_id = "en_0530926"

        facets = [
            Facet(
                name="color",
                type=FacetType.COUNT,
            ),
            Facet(
                name="size",
                type=FacetType.COUNT,
                values=["sm", "md"],
            ),
        ]

        # When
        result = client.more_like_this_search(
            document_id=document_id,
            collection_id=collection_id,
            facets=facets,
            account_id=account_params["id"],
        )

        # Then
        assert result.status == 200
        assert len(result.results) == 3
        assert len(result.facets) == len(facets)

    def test_more_like_these_search_with_facets(
        self,
        client: VantageClient,
        account_params: dict,
        test_collection_id: str,
    ):
        """
        Tests if MoreLikeThese search will return correct result with facets.
        """
        # Given
        collection_id = test_collection_id
        these = [
            MoreLikeTheseItem(
                weight=1.0,
                query_text="some text",
            ),
            MoreLikeTheseItem(
                weight=1.0,
                query_text="other text",
            ),
        ]

        facets = [
            Facet(
                name="color",
                type=FacetType.COUNT,
            ),
            Facet(
                name="size",
                type=FacetType.COUNT,
                values=["sm", "md"],
            ),
        ]

        # When
        result = client.more_like_these_search(
            more_like_these=these,
            collection_id=collection_id,
            facets=facets,
            account_id=account_params["id"],
        )

        # Then
        assert result.status == 200
        assert len(result.results) == 3
        assert len(result.facets) == len(facets)

    # endregion

    # region Additional

    def test_vantage_vibe_search(
        self,
        client: VantageClient,
        account_params: dict,
        test_collection_id: str,
    ):
        """
        Tests if Vantage Vibe search will return correct result.
        """
        # Given
        collection_id = test_collection_id
        vibe_id = "test-vibe-id"
        test_images = [
            VantageVibeImageUrl(url="https://www.someimageurl.com"),
            VantageVibeImageBase64(base64="imagebase64"),
        ]
        test_text = "test search"

        # When
        response = client.vantage_vibe_search(
            vibe_id=vibe_id,
            collection_id=collection_id,
            images=test_images,
            text=test_text,
            account_id=account_params["id"],
        )

        # Then
        assert response is not None
        assert response.status == 200
        assert len(response.results) == 3

    # endregion



# coding: utf-8

"""
    Vantage API

    This is a the API to interact with Vantage Discovery, the amazing Semantic Search Platform in the world.  We enable developers to build magical discovery experiences into their products and websites.  Some useful links: - [TODO: Semantic Search Guide: What Is It And Why Does It Matter?](https://www.bloomreach.com/en/blog/2019/semantic-search-explained-in-5-minutes)

    The version of the OpenAPI document: v1.1.2
    Contact: devrel@vantagediscovery.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import Any, ClassVar, Dict, List, Optional

from pydantic import (
    BaseModel,
    Field,
    StrictBool,
    StrictInt,
    StrictStr,
    field_validator,
)

from vantage_sdk.core.http.models.secondary_external_account import (
    SecondaryExternalAccount,
)


try:
    from typing import Self
except ImportError:
    from typing_extensions import Self


class Collection(BaseModel):
    """
    Collection
    """  # noqa: E501

    collection_created_time: Optional[StrictStr] = None
    collection_status: Optional[StrictStr] = None
    collection_state: Optional[StrictStr] = None
    collection_id: Optional[StrictStr] = Field(
        default=None,
        description="Immutable.  Unique identifier within an account, 3 to 36 characters long with only lower case letters, numeric digits and \"-\"",
    )
    user_provided_embeddings: Optional[StrictBool] = Field(
        default=False,
        description="Ignore llm field will provide own embeddings for both ingest and search",
    )
    llm: Optional[StrictStr] = None
    llm_provider: Optional[StrictStr] = None
    llm_secret: Optional[StrictStr] = None
    external_url: Optional[StrictStr] = None
    embeddings_dimension: Optional[StrictInt] = Field(
        default=None,
        description="The dimensionality or vector size of the embeddings.  Applies to both user provided embeddings and vantage managed embeddings.",
    )
    external_key_id: Optional[StrictStr] = Field(
        default=None,
        description="The external key, for the llm_provider to use for the collection",
    )
    secondary_external_accounts: Optional[
        List[SecondaryExternalAccount]
    ] = None
    collection_name: Optional[StrictStr] = None
    __properties: ClassVar[List[str]] = [
        "collection_created_time",
        "collection_status",
        "collection_state",
        "collection_id",
        "user_provided_embeddings",
        "llm",
        "llm_provider",
        "llm_secret",
        "external_url",
        "embeddings_dimension",
        "external_key_id",
        "secondary_external_accounts",
        "collection_name",
    ]

    @field_validator('collection_status')
    def collection_status_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in (
            'Pending',
            'Indexing',
            'Online',
            'Degraded',
            'Offline',
            'Empty',
        ):
            raise ValueError(
                "must be one of enum values ('Pending', 'Indexing', 'Online', 'Degraded', 'Offline', 'Empty')"
            )
        return value

    @field_validator('collection_state')
    def collection_state_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in ('Active', 'Standby', 'Deleted'):
            raise ValueError(
                "must be one of enum values ('Active', 'Standby', 'Deleted')"
            )
        return value

    @field_validator('llm_provider')
    def llm_provider_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in ('Hugging', 'OpenAI'):
            raise ValueError(
                "must be one of enum values ('Hugging', 'OpenAI')"
            )
        return value

    model_config = {
        "populate_by_name": True,
        "validate_assignment": True,
        "protected_namespaces": (),
    }

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of Collection from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        """
        _dict = self.model_dump(
            by_alias=True,
            exclude={
                "collection_created_time",
                "collection_status",
                "collection_state",
            },
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in secondary_external_accounts (list)
        _items = []
        if self.secondary_external_accounts:
            for _item in self.secondary_external_accounts:
                if _item:
                    _items.append(_item.to_dict())
            _dict['secondary_external_accounts'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: Dict) -> Self:
        """Create an instance of Collection from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "collection_created_time": obj.get("collection_created_time"),
                "collection_status": obj.get("collection_status"),
                "collection_state": obj.get("collection_state"),
                "collection_id": obj.get("collection_id"),
                "user_provided_embeddings": obj.get("user_provided_embeddings")
                if obj.get("user_provided_embeddings") is not None
                else False,
                "llm": obj.get("llm"),
                "llm_provider": obj.get("llm_provider"),
                "llm_secret": obj.get("llm_secret"),
                "external_url": obj.get("external_url"),
                "embeddings_dimension": obj.get("embeddings_dimension"),
                "external_key_id": obj.get("external_key_id"),
                "secondary_external_accounts": [
                    SecondaryExternalAccount.from_dict(_item)
                    for _item in obj.get("secondary_external_accounts")
                ]
                if obj.get("secondary_external_accounts") is not None
                else None,
                "collection_name": obj.get("collection_name"),
            }
        )
        return _obj



"""
Models for the Keys API.
"""

from enum import Enum
from typing import Optional

from pydantic import BaseModel, StrictStr


class VantageAPIKey(BaseModel):
    """
    Key for accessing Vantage API.

    Attributes
    ----------
    vantage_api_key_id: Optional[StrictStr]
        The unique identifier of the Vantage API key.
    account_id : Optional[str], optional
        The unique identifier of the account for which the Vantage API key is associated.
    vantage_api_key_obfuscated: Optional[StrictStr]
        Obfuscated value of the Vantage API key.
    status: Optional[StrictStr]
        Key status.
    """

    vantage_api_key_id: Optional[StrictStr] = None
    account_id: Optional[StrictStr] = None
    vantage_api_key_created_date: Optional[StrictStr] = None
    vantage_api_key_obfuscated: Optional[StrictStr] = None
    status: Optional[StrictStr] = None


class LLMProvider(Enum):
    """Supported LLM providers."""

    HuggingFace = "Hugging"
    OpenAI = "OpenAI"


class ExternalKey(BaseModel):
    """
    Key for accessing one of the external APIs.

    Attributes
    ----------
    external_key_id: Optional[StrictStr]
        The unique identifier of the external API key.
    account_id: Optional[StrictStr]
        The unique identifier of the account for which the Vantage API key is associated.
    external_key_created_date: Optional[StrictStr]
        Date when the key was created in the user account.
    llm_provider: Optional[StrictStr]
        For which LLM provider this key is intended.
    llm_secret: Optional[StrictStr]
        External key secret value.
    state: Optional[StrictStr]
        External key state.
    """

    external_key_id: Optional[StrictStr] = None
    account_id: Optional[StrictStr] = None
    external_key_created_date: Optional[StrictStr] = None
    llm_provider: Optional[StrictStr] = None
    llm_secret: Optional[StrictStr] = None
    state: Optional[StrictStr] = None


class SecondaryExternalAccount(BaseModel):
    """
    Secondary external account data.

    Attributes
    ----------
    external_account_id: Optional[StrictStr]
        Unique identifier of the external account.
    external_type: Optional[StrictStr]
        Type of the external account.
    """

    external_account_id: Optional[StrictStr] = None
    external_type: Optional[StrictStr] = None



"""This module contains ManagementAPI, a class for accessing management API."""

from __future__ import annotations

from vantage_sdk.core.http.api.account_management_api import (
    AccountManagementApi,
)
from vantage_sdk.core.http.api.collection_management_api import (
    CollectionManagementApi,
)
from vantage_sdk.core.http.api.documents_api import DocumentsApi
from vantage_sdk.core.http.api.external_keys_api import ExternalKeysApi
from vantage_sdk.core.http.api.shopping_assistant_api import (
    ShoppingAssistantApi,
)
from vantage_sdk.core.http.api.vantage_api_keys_api import VantageAPIKeysApi
from vantage_sdk.core.http.api.vantage_vibe_api import VantageVibeApi
from vantage_sdk.core.http.api_client import ApiClient


class ManagementAPI:
    """
    Class for accessing Management API.

    Attributes
    ----------
    account_api: AccountManagementApi
        Account management API component.
    collection_api: CollectionManagementApi
        Collection management API component.
    external_api_keys_api: ExternalAPIKeysApi
        External API keys management API component.
    vantage_api_keys_api: VantageAPIKeysApi
        Vantage API keys management API component.
    shopping_assistant_api: ShoppingAssistantApi
        Shopping Assistant management API component.
    vantage_vibe_api: ShoppingAssistantApi
        Vantage Vibe management API component.
    documents_api: DocumentsApi
        Documents management API component.
    """

    def __init__(
        self,
        account_api: AccountManagementApi,
        collection_api: CollectionManagementApi,
        external_keys_api: ExternalKeysApi,
        vantage_api_keys_api: VantageAPIKeysApi,
        shopping_assistant_api: ShoppingAssistantApi,
        vantage_vibe_api: VantageVibeApi,
        documents_api: DocumentsApi,
    ):
        """
        Default constructor.

        Parameters
        ----------
        account_api: AccountManagementApi
            Account management API component.
        collection_api: CollectionManagementApi
            Collection management API component.
        external_api_keys_api: ExternalAPIKeysApi
            External API keys management API component.
        vantage_api_keys_api: VantageAPIKeysApi
            Vantage API keys management API component.
        shopping_assistant_api: ShoppingAssistantApi
            Shopping Assistant management API component.
        vantage_vibe_api: ShoppingAssistantApi
            Vantage Vibe management API component.
        documents_api: DocumentsApi
            Documents management API component.
        """
        self.account_api = account_api
        self.collection_api = collection_api
        self.external_keys_api = external_keys_api
        self.vantage_api_keys_api = vantage_api_keys_api
        self.shopping_assistant_api = shopping_assistant_api
        self.vantage_vibe_api = vantage_vibe_api
        self.documents_api = documents_api

    @classmethod
    def from_defaults(cls, api_client: ApiClient) -> ManagementAPI:
        """
        Constructs ManagementAPI instance using default values.

        This method is the preferred way to construct ManagementAPI instance.

        Parameters
        ----------
        api_client: ApiClient
            API client component used to call the API.

        Returns
        -------
        ManagementAPI
            ManagementAPI instance.
        """
        account_api = AccountManagementApi(api_client=api_client)
        collection_api = CollectionManagementApi(api_client=api_client)
        external_keys_api = ExternalKeysApi(api_client=api_client)
        vantage_api_keys_api = VantageAPIKeysApi(api_client=api_client)
        shopping_assistant_api = ShoppingAssistantApi(api_client=api_client)
        vantage_vibe_api = VantageVibeApi(api_client=api_client)
        documents_api = DocumentsApi(api_client=api_client)
        return cls(
            account_api=account_api,
            collection_api=collection_api,
            external_keys_api=external_keys_api,
            vantage_api_keys_api=vantage_api_keys_api,
            shopping_assistant_api=shopping_assistant_api,
            vantage_vibe_api=vantage_vibe_api,
            documents_api=documents_api,
        )
